/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * Transport mechanism benchmarking application.
 * FIXME: add options to toggle between gnuplot and human-readable
 * output format.
 *
 * @author Paul Ruth
 * @file applications/tbench/gnunet-tbench.c 
 **/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "util/port.h"
#include "applications/common/printhelp.h"
#include "applications/tbench/tbench.h"
#include "util/getopt.h"
#include "util/configuration.h"
#include "util/logging.h"
#include "util/initialize.h"
#include "util/returnvalues.h"
#include "util/xmalloc.h"

#define TBENCH_VERSION "0.0.2"

#define DEFAULT_MESSAGE_SIZE	10
#define DEFAULT_TIMEOUT		2
#define DEFAULT_SPACING		0
/* */
static int  messageSize = DEFAULT_MESSAGE_SIZE;
static int  messageCnt  = 1;
static char messageReceiver[256];
static int  messageIterations = 1;
static int  messageTrainSize = 1;
static int  messageTimeOut = DEFAULT_TIMEOUT;
static int  messageSpacing = DEFAULT_SPACING;
/* */

static void tbenchUsage() {
  printf("-r RECIEVER, --rec=RECIEVER   : "
	 "reciever host num (hex file name)\n"
	 "-n MESSAGES, --msg=MESSAGES   : "
	 "number of messages per iteration\n"
	 "-s SIZE, --size=SIZE          : "
	 "message size (default: %d)\n"
	 "-i ITER, --iterations=ITER    : "
	 "number of iterations\n"
	 "-t TIMEOUT, --timeout=TIMEOUT : "
	 "time to wait for the arrival of a response (default: %d ms)\n"
	 "-S SPACE, --space=SPACE       : "
	 "inter train message spacing (default: %d ms)\n"
	 "-X COUNT, --xspace=COUNT      : "
	 "sleep for SPACE ms after COUNT messages\n",
	 DEFAULT_MESSAGE_SIZE,
	 DEFAULT_TIMEOUT,
	 DEFAULT_SPACING
	 );
}
/**
 * Parse the options, set the timeout.
 * @param argc the number of options
 * @param argv the option list (including keywords)
 * @return OK on error, SYSERR if we should exit 
 **/
static int parseOptions(int argc,
			char ** argv) {
  int option_index;
  int c;  

  FREENONNULL(setConfigurationString("GNUNETD",
				     "LOGFILE",
				     NULL));
  while (1) {
    static struct GNoption long_options[] = {
      LONG_DEFAULT_OPTIONS,
      { "nickname", 1, 0, 'n' },
      { 0,0,0,0 }
    };    
    option_index=0;
    c = GNgetopt_long(argc,
		      argv, 
		      "vhdc:L:H:n:s:n:r:i:t:S:X:", 
		      long_options, 
		      &option_index);    
    if (c == -1) 
      break;  // No more flags to process
    if (YES == parseDefaultOptions(c, optarg))
      continue;
    switch(c) {
    case 'v': 
      printf("GNUnet v%s, gnunet-tbench v%s\n",
	     VERSION,
	     TBENCH_VERSION);
      return SYSERR;

    case 'h': 
      printf("USAGE: gnunet-tbench\n");
      printHelpDefault();
      tbenchUsage();
      return SYSERR;

    case 's': 
      if(!sscanf(optarg,"%d",&messageSize)){
	printf("-s argument not a number\n");
	exit(1);
      }
      break;

    case 'X':
      if(!sscanf(optarg,"%d",&messageTrainSize)){
	printf("-X argument not a number\n");
	exit(1);
      }
      break;
    case 'n': 
      if(!sscanf(optarg,"%d",&messageCnt)){
	printf("-n argument not a number\n");
	exit(1);
      }
      break;

    case 'r': 
      strncpy(messageReceiver, optarg, strlen(optarg));
      break;

    case 'i': 
      if(!sscanf(optarg,"%d",&messageIterations)){
	printf("-i argument not a number\n");
	exit(1);
      }
      break;

    case 't':
      if(!sscanf(optarg,"%d",&messageTimeOut)){
	printf("-t argument not a number\n");
	exit(1);
      }
      break;

    case 'S':
      if(!sscanf(optarg,"%d",&messageSpacing)){
	printf("-S argument not a number\n");
	exit(1);
      }
      break;

    default: 
      LOG(LOG_FAILURE,
	  "FAILURE: Unknown option %c. Aborting.\n"\
	  "Use --help to get a list of options.\n",
	  c);
      return -1;
    } /* end of parsing commandline */
  } /* while (1) */
  return OK;
}
/* */
/**
 * The main function to search for files on GNet.
 * @param argc number of arguments from the command line
 * @param argv command line arguments
 * @return return value from gnunetsearch: 0: ok, -1: error
 **/   
int main(int argc, char ** argv) {
  GNUNET_TCP_SOCKET * sock;
  TBENCH_CS_MESSAGE msg;
  TBENCH_CS_REPLY * buffer;
  float messagesPercentLoss;

  if (SYSERR == initUtil(argc, argv, &parseOptions))
    return 0; /* parse error, --help, etc. */ 
  sock = getClientSocket();

  memset(&msg,
	 0,
	 sizeof(TBENCH_CS_MESSAGE));
  msg.msgSize     =htons(messageSize);
  msg.msgCnt      =htons(messageCnt);
  msg.iterations  =htons(messageIterations);
  msg.intPktSpace =htons(messageSpacing);
  msg.trainSize   =htons(messageTrainSize);
  msg.timeOut     =htonl(messageTimeOut);
  if (strlen(messageReceiver) != 2*sizeof(HashCode160))
    errexit("Invalid receiver peer ID specified (%s is not %d character hex name)\n",
	    messageReceiver,
	    2*sizeof(HashCode160));
  hex2hash((HexName*)messageReceiver,
	   &msg.receiverId.hashPubKey);
  msg.header.size = htons(sizeof(TBENCH_CS_MESSAGE));
  msg.header.tcpType = htons(TBENCH_CS_PROTO_REQUEST);

  if (SYSERR == writeToSocket(sock,
			      &msg.header))
    return -1;
  
  buffer = MALLOC(MAX_BUFFER_SIZE);
  LOG(LOG_DEBUG,
      "DEBUG: reading from readFromSocket\n");
  if (OK == readFromSocket(sock, (CS_HEADER**)&buffer)) {
    if((float)buffer->mean_loss <= 0){
      messagesPercentLoss = 0.0;
    } else {
      messagesPercentLoss = (buffer->mean_loss/((float)htons(msg.msgCnt)));
    }
    /*
    printf("\n#-s %d, n %d, i %d, t %d, -S %d, -X %d\n",
	   htons(msg.msgSize),
	   htons(msg.msgCnt),
	   htons(msg.iterations),
	   htons(msg.timeOut),
	   htons(msg.intPktSpace),
	   htons(msg.trainSize));
    */ 
    printf("%f %f\n",buffer->mean_time,1.0-messagesPercentLoss);
    
    /*
      printf("Time:\n");
      printf("\tmax      %d\n",htons(buffer->max_time));
      printf("\tmin      %d\n",htons(buffer->min_time));
      printf("\tmean     %f\n",buffer->mean_time);
      printf("\tvariance %f\n",buffer->variance_time);
      
      printf("Loss:\n");
      printf("\tmax      %d\n",htons(buffer->max_loss));
      printf("\tmin      %d\n",htons(buffer->min_loss));
      printf("\tmean     %f\n",buffer->mean_loss);
      printf("\tvariance %f\n",buffer->variance_loss); 
    */
  } else 
    printf("\nDid not receive the message from gnunetd. Is gnunetd running?\n");  
  FREE(buffer);

  releaseClientSocket(sock);
  doneUtil();
  return 0;
}
/* end of gnunet-tbench.c */ 
