/*
     This file is part of GNUnet.
     (C) 2001, 2002 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/
/**
 * @file include/server/traffic.h
 * @author Christian Grothoff
 * 
 * Module to keep track of recent amounts of p2p traffic
 * on the local GNUnet node.
 **/
#ifndef TRAFFIC_H
#define TRAFFIC_H

#include "server/coreapi.h"
/**
 * Initialize the traffic module.
 **/
void initTraffic();

/**
 * Shutdown the traffic module.
 **/
void doneTraffic();

/**
 * A message was received. Update traffic stats.
 * @param header the header of the message
 * @param sender the identity of the sender 
 **/
void trafficReceive(p2p_HEADER * header,
		    HostIdentity * sender);

/**
 * A message is send. Update traffic stats.
 * @param header the header of the message
 **/
void trafficSend(p2p_HEADER * header,
		 HostIdentity * receiver);

/**
 * This type is for messages that we send.
 **/
#define TC_SENT      0x8000

/**
 * This type is for messages that we receive.
 **/
#define TC_RECEIVED  0x4000

#define TC_TYPE_MASK (TC_RECEIVED|TC_SENT)

/**
 * From/To how many different peers did we receive/send
 * messages of this type? (bitmask)
 **/
#define TC_DIVERSITY_MASK 0xFFF

/**
 * Counter for traffic.
 **/
typedef struct {

  /**
   * Flags. See TC_XXXX definitions.
   **/
  unsigned short flags;

  /**
   * What was the number of messages of this type that the peer
   * processed in the last n time units?
   **/
  unsigned short count;

  /**
   * What is the message type that this counter is concerned with?
   **/
  unsigned short type;

  /**
   * What is the average size of the last "count" messages that
   * the peer processed?
   **/
  unsigned short avrg_size;

  /**
   * In which of the last 32 time units did the peer receive or send a
   * message of this type? The lowest bit (1) corresponds to -31
   * seconds ago, the highest bit (2^31) corresponds to the current
   * second.
   **/
  unsigned int time_slots;
} TRAFFIC_COUNTER;

/**
 * Format of the reply-message to a CS_TRAFFIC_QUERY.
 * A message of this format is send back to the client
 * if it sends a CS_TRAFFIC_QUERY to gnunetd.
 **/
typedef struct {
  CS_HEADER header;

  /**
   * The number of different message types we have seen
   * in the last time.
   **/
  unsigned int count;

  /**
   * "count" traffic counters.
   **/
  TRAFFIC_COUNTER counters[0];

} CS_TRAFFIC_INFO;

/**
 * Request for CS_TRAFFIC_INFO.
 **/
typedef struct {
  CS_HEADER header;

  /**
   * How many time units back should the statistics returned contain? 
   * (in network byte order) Must be smaller or equal to HISTORY_SIZE.
   **/
  unsigned int timePeriod;

} CS_TRAFFIC_REQUEST;

#/**
 * What is the unit of time (in cron_t) for the traffic module? This
 * constant essentially specifies the resolution of the distribution
 * function that is applied for sampling traffic. Default is 1s.
 **/ 
#define TRAFFIC_TIME_UNIT cronSECONDS

/**
 * How many time-units back do we keep the history of?  (must really
 * be <=32 since we use the 32 bit in an unsigned int). The memory
 * impact of this value n is 4 * 3 * MAX_MESSAGE_ID * n, which is for
 * the default of n=32 with the current MAX_MESSAGE_ID being roughly a
 * dozen less than 2k.
 **/
#define HISTORY_SIZE 32

/**
 * Get statistics over the number of messages that
 * were received or send of a given type.
 *
 * @param messageType the type of the message
 * @param sendReceive TC_SEND for sending, TC_RECEIVE for receiving
 * @param timePeriod how many TRAFFIC_TIME_UNITs to take
 *        into consideration (limited by HISTORY_SIZE)
 * @param avgMessageSize average size of the messages (set)
 * @param messageCount number of messages (set)
 * @param peerCount number of peers engaged (set)
 * @param timeDistribution bit-vector giving times of interactions,
 *        highest bit is current time-unit, bit 1 is 32 time-units ago (set)
 * @return OK on success, SYSERR on error
 **/
int getTrafficStats(const unsigned short messageType,
		    const int sendReceive,
		    const unsigned int timePeriod,
		    unsigned short * avgMessageSize,
		    unsigned short * messageCount,
		    unsigned int * peerCount,
		    unsigned int * timeDistribution);


#endif
/* end of traffic.h */
