/*
     This file is part of GNUnet.
     (C) 2001, 2002, 2003 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * Test for the transports. This utility can be used to test if 
 * a transport mechanism for GNUnet is properly configured.
 *
 * @author Christian Grothoff
 * @file server/gnunet-transport-check.c
 **/

#include "config.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>

#include <netdb.h>
#include <string.h>
#include <unistd.h>

#include "server/transport.h"
#include "util/configuration.h"
#include "util/getopt.h"
#include "util/initialize.h"
#include "util/logging.h"
#include "util/returnvalues.h"
#include "util/xmalloc.h"

#define TEST_CRC 25116123
#define DEFAULT_MSG "Hello World"

static char * testmsg;

static MessagePack * receit;
static CoreAPIForTransport capi;
static Semaphore * sem;

static void receive(MessagePack * mp) {
  receit = mp;
  SEMAPHORE_UP(sem);
}

static void semUp(Semaphore * sem) {
  SEMAPHORE_UP(sem);
}

/**
 * Test the given transport API.
 **/
static void testTAPI(TransportAPI * tapi,
		     int * res) {
  HELO_Message * helo;
  TSession * tsession;
  unsigned int repeat;
  cron_t start;
  cron_t end;

  if (tapi == NULL)
    errexit("Could not initialize transport!\n");

  helo = NULL;
  if (OK != tapi->startTransportServer()) {
    fprintf(stderr,
	    "ERROR: could not start transport server\n");
    *res = SYSERR;
    return;
  }
  if (OK != tapi->createHELO(&helo)) {
    fprintf(stderr,
	    "ERROR: could not create HELO\n");
    *res = SYSERR;
    return;
  }
  if (OK != tapi->connect(helo,
			  &tsession)) {
    fprintf(stderr,
	    "ERROR: could not connect\n");
    *res = SYSERR;
    tapi->stopTransportServer();
    FREE(helo);
    return;
  }
  repeat = getConfigurationInt("TRANSPORT-CHECK",
			       "REPEAT");
  if (repeat == 0)
    repeat = 1;
  sem = SEMAPHORE_NEW(0);
  cronTime(&start);
  while (repeat > 0) {
    repeat--;
    receit = NULL;
    if (OK != tapi->send(tsession,
			 testmsg,
			 strlen(testmsg),
			 NO,
			 TEST_CRC)) {
      fprintf(stderr,
	      "ERROR: could not send\n");
      *res = SYSERR;
      tapi->disconnect(tsession);
      tapi->stopTransportServer();
      SEMAPHORE_FREE(sem);
      return;
    }
    addCronJob((CronJob)&semUp,
	       1 * cronMINUTES,
	       0,
	       sem);
    SEMAPHORE_DOWN(sem);
    suspendCron();
    delCronJob((CronJob)&semUp,
	       0,
	       sem); 
    resumeCron();
    if (receit == NULL) {
      fprintf(stderr,
	      "ERROR: did not receive message within 60 seconds.\n");
      *res = SYSERR;
      tapi->disconnect(tsession);
      tapi->stopTransportServer();
      SEMAPHORE_FREE(sem);
      return;
    }
    if ( (receit->crc != TEST_CRC) ||
	 (receit->size != strlen(testmsg)) ||
	 (receit->isEncrypted != NO) ||
	 (0 != memcmp(capi.myIdentity,
		      &receit->sender,
		      sizeof(HostIdentity))) ||
	 (0 != memcmp(receit->msg,
		      testmsg,
		      strlen(testmsg))) ) {
      fprintf(stderr,
	      "ERROR: message received was invalid\n");
      *res = SYSERR;
      tapi->disconnect(tsession);
      tapi->stopTransportServer();
      SEMAPHORE_FREE(sem);
      return;
    }
    FREE(receit->msg);
    FREE(receit);
  }
  cronTime(&end);
  if (OK != tapi->disconnect(tsession)) {
    fprintf(stderr,
	    "ERROR: could not disconnect\n");
    *res = SYSERR;
    tapi->stopTransportServer();
    SEMAPHORE_FREE(sem);
    return;
  }
  if (OK != tapi->stopTransportServer()) {
    fprintf(stderr,
	    "ERROR: could not stop server\n");
    *res = SYSERR;
    SEMAPHORE_FREE(sem);
    return;
  }

  SEMAPHORE_FREE(sem);
  printf("Transport OK, %ums for %d messages of size %d bytes.\n",
	 (unsigned int) ((end - start)/cronMILLIS),  
	 getConfigurationInt("TRANSPORT-CHECK",
			     "REPEAT"),
	 strlen(testmsg));
}

/**
 * Perform option parsing from the command line. 
 **/
static int parser(int argc, 
	   char * argv[]) {
  int cont = OK;
  int c;


  FREENONNULL(setConfigurationString("GNUNETD",
				     "LOGFILE",
				     NULL));
  while (1) {
    int option_index = 0;
    static struct GNoption long_options[] = {
      { "loglevel",1, 0, 'L' },
      { "config",  1, 0, 'c' },
      { "version", 0, 0, 'v' },
      { "help",    0, 0, 'h' },
      { "transport", 1, 0, 't' },
      { "repeat",  1, 0, 'r' },
      { "size", 1, 0, 's'},
      { "Xrepeat", 1, 0, 'x' },
      { "Xport", 1, 0, 'P' },
      { 0,0,0,0 }
    };
    
    c = GNgetopt_long(argc,
		      argv, 
		      "vhc:L:t:r:s:X:P:", 
		      long_options, 
		      &option_index);
    
    if (c == -1) 
      break;  // No more flags to process
    
    switch(c) {
    case 'P':{
      unsigned int port;
      if (1 != sscanf(optarg, "%ud", &port)) {
	LOG(LOG_FAILURE, 
	    "You must pass a number to the -P option.\n");
	return SYSERR;
      } else {
	setConfigurationInt("TCP", "PORT", port);
	setConfigurationInt("UDP", "PORT", port);
      }
      break;
    }
    case 's':{
      unsigned int size;
      if (1 != sscanf(optarg, "%ud", &size)) {
	LOG(LOG_FAILURE, 
	    "You must pass a number to the -s option.\n");
	return SYSERR;
      } else {
	if (size == 0)
	  size = 1;
	else
	  size++;
	testmsg = malloc(size);
	testmsg[--size] = '\0';
	while (size > 0)
	  testmsg[--size] = 'A';
      }
      break;
    }
    case 'r':{
      unsigned int repeat;
      if (1 != sscanf(optarg, "%ud", &repeat)) {
	LOG(LOG_FAILURE, 
	    "You must pass a number to the -r option.\n");
	return SYSERR;
      } else {
	setConfigurationInt("TRANSPORT-CHECK",
			    "REPEAT",
			    repeat);
      }
      break;
    }
    case 'X':{
      unsigned int repeat;
      if (1 != sscanf(optarg, "%ud", &repeat)) {
	LOG(LOG_FAILURE, 
	    "You must pass a number to the -X option.\n");
	return SYSERR;
      } else {
	setConfigurationInt("TRANSPORT-CHECK",
			    "X-REPEAT",
			    repeat);
      }
      break;
    }
    case 'c':
      FREENONNULL(setConfigurationString("FILES",
					 "gnunet.conf",
					 optarg));
      break;
    case 't':
      FREENONNULL(setConfigurationString("GNUNETD",
					 "TRANSPORTS",
					 optarg));
      break;
    case 'v': 
      printf("gnunet-transport-check v%s\n",
	     VERSION);
      cont = SYSERR;
      break;
    case 'h': 
      printf("USAGE: gnunet-transport-check [OPTIONS]\n"\
	     "-h, --help                    : "
	     "print this page\n"
	     "-c FILENAME, --config=FILENAME: "
	     "load config file (defaults: %s)\n"
	     "-v, --version                 : "
	     "print the version number\n"\
	     "-t TRANS, --transport=TRANS   : "
	     "test transport TRANS\n"\
	     "-L LOGLEVEL, --loglevel=LOGL  : "
	     "set the loglevel\n"
	     "-r COUNT, --repeat=COUNT      : "
	     "send COUNT messages (default: 1)\n"
	     "-s SIZE, --size=SIZE          : "
	     "send messages with SIZE bytes payload (default: %d)\n",
	     DEFAULT_CONFIG_FILE,
	     strlen(DEFAULT_MSG));
      cont = SYSERR;
      break;
    case 'L':
      FREENONNULL(setConfigurationString("GNUNETD",
					 "LOGLEVEL",
					 optarg));
      break;
    default:
      LOG(LOG_FAILURE, 
	  "FAILURE: Unknown option %c. Aborting.\n"\
	  "Use --help to get a list of options.\n",
	  c);
      cont = SYSERR;    
    } /* end of parsing commandline */
  }
  if (optind < argc) {
    LOG(LOG_WARNING, 
	"WARNING: Invalid arguments: ");
    while (optind < argc)
      LOG(LOG_WARNING, 
	  "%s ", argv[optind++]);
    LOG(LOG_FATAL,
	"FATAL: Invalid arguments. Exiting.\n");
    return SYSERR;
  }
  return cont;
}

CoreAPIForTransport * getCoreAPIForTransport() {
  return &capi;
}

int main(int argc, char *argv[]) {
  HostIdentity myIdentity;
  int res;
  int Xrepeat;
  
  testmsg = DEFAULT_MSG;
  if (OK != initUtil(argc, argv, &parser))
    return SYSERR;
  printf("Testing transport(s) %s\n",
	 getConfigurationString("GNUNETD",
				"TRANSPORTS"));
  initTransports();
  startCron();

  capi.version = 0;
  capi.receive = &receive;
  capi.myIdentity = &myIdentity;
  Xrepeat = getConfigurationInt("TRANSPORT-CHECK",
				"X-REPEAT");
  if (Xrepeat == 0)
    Xrepeat = 1;
  res = OK;
  while (Xrepeat-- > 0) {
    forEachTransport((TransportCallback)&testTAPI,
		     &res);
  }
  stopCron();
    doneTransports();
  doneUtil();
  if (res == OK)
    return 0;
  else
    return -1;
}


